/**
 *
 * \file     kml_plugin_if.h
 * \brief    kmath lib API plugin module interface routines
 * \authoers Toshiyuki Imamura (TI)
 * \date     2015/02/02 (NT)
 * 
 * (c) Copyright 2014 RIKEN. All rights reserved.
 */

#ifndef __kml_plugin_if_h__
#define __kml_plugin_if_h__

/**
 *
 * Function    regist_plugin_routines_
 * \brief      plugin module register routine
 * \authors    NT
 * \date       2015/02/02 (NT)
 *
 */
void kml_regist_plugin_routines_(long long* handle, 
								 long long* pcontext,
								 const char* in_parameters,
								 const char* out_parameters,
								 const char* mem_parameters,
								 void* func_initialize,
								 void* func_shutdown,
								 void* func_realloc,
								 void* func_solve,
								 void* func_to_internal,
								 void* func_to_standard,
								 void* func_reuse_internal,
								 int in_param_len,
								 int out_param_len,
								 int mem_param_len);

/**
 *
 * Function    get_mpi_communicator_
 * \brief      get MPI Communicator
 * \authors    NT
 * \date       2015/02/09 (NT)
 *
 */
void kml_get_mpi_communicator_(long long* handle, int* comm);

/**
 *
 * Function    get_parameter_int_
 * \brief      get integer parameter value
 * \authors    NT
 * \date       2015/02/02 (NT)
 *
 */
void kml_get_parameter_int_(long long* handle, 
							const char* key, int* val, int* err,
							int key_len);

/**
 *
 * Function    get_parameter_real_
 * \brief      get real parameter value
 * \authors    NT
 * \date       2015/02/02 (NT)
 *
 */
void kml_get_parameter_real_(long long* handle,
							 const char* key, double* val, int* err,
							 int key_len);

/**
 *
 * Function    get_parameter_string_
 * \brief      get string parameter value
 * \authors    NT
 * \date       2015/02/02 (NT)
 *
 */
void kml_get_parameter_string_(long long* handle,
							   const char* key, char* val, int* err, 
							   int key_len, int val_len);

/**
 *
 * Function    get_parameter_petsc_
 * \brief      get PETSc parameter value
 * \authors    NT
 * \date       2015/02/02 (NT)
 *
 */
void kml_get_parameter_petsc_(long long* handle,
							  const char* key, long long* val64, int* err,
							  int key_len);

#endif // !__kml_plugin_if_h__
