program test

  use kmath_lib_mod
  use kmath_plugin_fft_fftw_mod
  use kmath_plugin_fft_kmath_fft3d_mod

  implicit none

#include <finclude/petscsys.h>
#include <finclude/petscvec.h>
#include <finclude/petscmat.h>

  Mat :: A
  Mat :: X
  Mat :: Y

  PetscScalar          :: v(1), alpha
  PetscReal            :: norm
  PetscReal            :: r1, r2
  PetscInt             :: ierr
  integer              :: sn
  integer              :: rank
  integer              :: m, n, o, i, j, k, iar(1), jar(1)
  integer, allocatable :: seed(:)
  type(s_context)      :: h


  call random_seed(size = sn)
  allocate(seed(sn))
  seed = 123;
  call random_seed(put = seed)
  deallocate(seed)

  call PetscInitialize(PETSC_NULL_CHARACTER, ierr)

  call MPI_Comm_rank(PETSC_COMM_WORLD,rank,ierr)

  ! create matrix
  m = 500
  n = 500
  o = 500

  call MatCreateDense(PETSC_COMM_WORLD,PETSC_DECIDE, &
                      PETSC_DECIDE, m*n, o, PETSC_NULL_SCALAR, A, ierr)
  call MatCreateDense(PETSC_COMM_WORLD,PETSC_DECIDE, &
                      PETSC_DECIDE, m*n, o, PETSC_NULL_SCALAR, X, ierr)
  call MatCreateDense(PETSC_COMM_WORLD,PETSC_DECIDE, &
                      PETSC_DECIDE, m*n, o, PETSC_NULL_SCALAR, Y, ierr)

  if (rank == 0) then
    do k = 0, o - 1
      jar(1) = k
      do j = 0, n - 1
        do i = 0, m - 1
          iar(1) = i + j * m
          call random_number(r1)
          call random_number(r2)
          v(1) = dcmplx(r1, r2)
          call MatSetValues(A, 1, iar, 1, jar, v, INSERT_VALUES, ierr)
        end do
      end do
    end do
  end if

  call MatAssemblyBegin(A, MAT_FINAL_ASSEMBLY, ierr)
  call MatAssemblyEnd  (A, MAT_FINAL_ASSEMBLY, ierr)

  call KMATH_Create(h, PETSC_COMM_WORLD, ierr)

  ! solve by KMATH API (with FFTW)
  call KMATH_Set_Plugin(h, KMATH_Plugin_Setup_FFT_FFTW, ierr)

  call KMATH_Set_Parameter(h, 'dimension', 3, ierr)
  call KMATH_Set_Parameters(h, (/'size_x','size_y','size_z'/), (/m,n,o/), ierr)

  call KMATH_Set_Parameter(h, 'direction', 'forward', ierr)
  call KMATH_Set_Parameters(h, (/'A','X'/), (/A,X/), ierr)
  call KMATH_Solve(h, ierr)

  ! solve by KMATH API (with KMATH FFT3D)
  call KMATH_Set_Plugin(h, KMATH_Plugin_Setup_FFT_KMATH_FFT3D, ierr)

  call KMATH_Set_Parameter(h, 'direction', 'backward', ierr)
  call KMATH_Set_Parameters(h, (/'A','X'/), (/X,Y/), ierr)
  call KMATH_Solve(h, ierr)

  call KMATH_Destroy(h, ierr)

  ! result check
  alpha = -1.0d0
  call MatAXPY(Y, alpha, A, DIFFERENT_NONZERO_PATTERN, ierr)

  call MatNorm(Y, NORM_FROBENIUS, norm, ierr)
  if(rank == 0) then
    print*,'diff(FROBENIUS NORM) = ', norm
  end if

  call MatDestroy(Y, ierr)
  call MatDestroy(X, ierr)
  call MatDestroy(A, ierr)

  call PetscFinalize(ierr)

  stop

end program test
